//
//  MCLogInfoToMIDIFile.h
//  MusiCOG
//
//  Created by James Maxwell on 2013-05-27.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioToolbox.h>


@class mcMusicPlayerController, mcMusiCogController, mcWMStream, mcEvent, mcWMSegment;

@interface MCLogInfoToMIDIFile : NSObject {
    mcMusiCogController* musiCog;
    MusicSequence logSequence;
    NSString* logInfoFileName;
    NSMutableDictionary* trackLogMap;
    NSMutableDictionary* segmentationDictionary;
}

@property (assign, nonatomic) mcMusiCogController* musiCog;
@property (assign, nonatomic) MusicSequence logSequence;
@property (retain, nonatomic) NSString* logInfoFileName;
@property (retain, nonatomic) NSMutableDictionary* trackLogMap;
@property (retain, nonatomic) NSMutableDictionary* segmentationDictionary;

+ (MCLogInfoToMIDIFile*) sharedInstance;
+ (MCLogInfoToMIDIFile*) sharedInstanceWithMusiCOG:(mcMusiCogController*)musiCOG;

- (void) setMusiCog:(/*__unsafe_unretained*/ mcMusiCogController *)mc;
- (mcMusiCogController*) musiCog;
- (BOOL) processLogInfoForStream:(mcWMStream*)aStream
                 logSegmentation:(BOOL)logSegments
                 logAssociations:(BOOL)logAssocs;
- (void) addEvent:(mcEvent*)anEvent toTrack:(MusicTrack)aTrack isBackground:(BOOL)flag durationEndEvent:(mcEvent*)endEvent;
- (MusicTrack) addNewTrackForChunk:(mcWMSegment*)aChunk inStream:(mcWMStream*)aStream;
- (void) setNameForTrack:(MusicTrack)aTrack withString:(NSString*)aString;
- (BOOL) saveLogSequenceToMIDIFile:(NSString*)fileName makeLegato:(BOOL)makeLeg;
- (BOOL) saveSegmentationDictionary:(NSDictionary*)segDict toTextFile:(NSString*)fileName;
- (void) clearLogInfoFileForNewRun;
- (void) clearLogInfoFileInRangeFromStart:(MusicTimeStamp)start toEnd:(MusicTimeStamp)end;

- (id) initWithMusiCOG:(mcMusiCogController*)mc;

@end

extern UInt8 kMIDIMetaEvent_Text; // = 0x01;	 // just text. any amount. should describe the track, however...
extern UInt8 kMIDIMetaEvent_TrackName; // = 0x03;	 // the name of the track
extern UInt8 kMIDIMetaEvent_SequencerSpecific; // = 0x7F;	 // sequencer- specific event
extern UInt8 kMIDIMetaEvent_Test; // = 0xFA;	 // dummy event used for self test methods

/**
 
 This class provides a simple method for automatically creating MIDI files containing info about MusiCog's internal state.
 For example, given an analysis MIDI file, at the end of learning, a copy of the analysis file can be exported with important
 info about streaming, hierarchical segmentation, etc. Info is copied out to different named tracks (e.g., "L1 segmentation"),
 making it easy to analyze. When used to record segmentation data, the boundary events at each level are copied to a new track,
 so that the exported logInfo MIDI file will contain the original melody on track 1, the L1 boundaries on track 2, the L2
 boundaries on track 3, and so on. For stream separation, the streams are copied to their own tracks, allowing easy visualization
 of the separation (and easy identification of problems like voice-crossings, and so on).
 It will also be possible to export logInfo files for associations. These will be encoded as sequences of "chords," where only the 
 associated nodes/events are recorded to tracks/streams (i.e., intervening notes are omitted).
 
*/