//
//  mcCMLevel.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-07.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>

@class mcCMModel, mcCMNode, mcCMEdge;

@interface mcCMLevel : NSObject

@property (weak, nonatomic) mcCMModel* model;
@property (assign, nonatomic) UInt8 levelNumber;
@property (assign, nonatomic) NSUInteger rootNodeID;
@property (strong, nonatomic) NSPointerArray* terminalNodes;    // sparse array: node index -> {edge 1, ..., edge n}
@property (strong, nonatomic) NSPointerArray* targetNodes;      // sparse array: node index -> {edge 1, ..., edge n}
@property (strong, nonatomic) NSPointerArray* superTierNodes;   // sparse array: node index -> {node 1, ..., node n}
@property (strong, nonatomic) NSMutableArray* uniqueTerminals;  // set of unique terminals -- cleared for each corpus
@property (assign, nonatomic) long* ordinalFormTransitions;   // transition matrix for ordinal form IDs

- (mcCMNode*) rootNode;
- (NSPointerArray*) nodesAdjacentToNode:(mcCMNode*)aNode;       // returns nodes adjacent to aNode
- (NSArray*) childrenOfNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;                   // returns array of nodes deeper than aNode
- (NSArray*) parentsOfNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;                    // returns array of nodes shallower than aNode
- (NSArray*) superTiersOfNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;                 // returns superTiers of aNode
- (NSArray*) terminalsOfNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;                  // returns terminals of aNode
- (NSArray*) targetsOfNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;                    // returns targets of aNode
- (mcCMEdge*) addChildNode:(mcCMNode*)childNode forNode:(mcCMNode*)parentNode;
- (void) addSuperTierNode:(mcCMNode*)superTier forNode:(mcCMNode*)aNode;
- (void) addTerminalNode:(mcCMNode*)terminalNode forNode:(mcCMNode*)aNode;
- (void) addTargetNode:(mcCMNode*)targetNode forNode:(mcCMNode*)aNode;
- (UInt32) countNodesOnTier:(UInt8)tier;
- (void) clearLevelData;
- (id) initWithModel:(mcCMModel*)aModel;

@end

/****
 
 Rather than storing NSUIntegers with the edgeID, I'm storing BOOLs at the index of the edgeID. This is just simpler,
 and takes less memory (i.e., a BOOL vs an NSUInteger).

 The use of NSPointerArrays here allows the node IDs to be stored as indices, not keys in dictionaries (like before), thus 
 allowing a sparse array representation. This way, I just use the node's ID directly as an index, so there's no searching.
 The returned array is an array of mcCMEdge pointers. 

**/