//
//  mcEvent.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-08.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioToolbox.h>

@class mcCMController, mcCMNode, mcCMModel;

typedef struct MCMusicEvent {
    UInt32 length;          // required by Core Audio's user event type
    UInt32 typeID;          // used by the MusicPlayer callback
    UInt16 trackID;         // used by the MusicPlayer callback -- Nov build: use as streamID
    Boolean status;         // used by the MusicPlayer callback
    UInt8 pitch;            // MIDI pitch (also Identity rep)
    UInt8 velocity;         // MIDI velocity
    UInt8 channel;          // 1-16
    MusicTimeStamp timestamp;// timestamp in beats
    MusicTimeStamp duration;// duration in beats
    long ms_duration;       // for max compatibility
} MCMusicEvent;

@interface mcEvent : NSObject <NSCopying>

@property (assign, nonatomic) mcCMController* controller;
@property (assign, nonatomic) MCMusicEvent* musicEvent;
@property (strong, nonatomic) mcEvent* prevEvent;
@property (assign, nonatomic) char schemaChar;      // this is used for generation, as a reference for the contour
@property (assign, nonatomic) char rSchemaChar;     // same idea (but I'm not sure this is necessary)
@property (assign, nonatomic) float segmentCohesion;
@property (assign, nonatomic) float pitchSalience;
@property (assign, nonatomic) float accentStrength; // pitch, duration, pulse, contour, interval
@property (assign, nonatomic) BOOL isLiveInput;
@property (assign, nonatomic) MusicSequence sequence;
@property (assign, nonatomic) UInt8 scaleStep;
@property (assign, nonatomic) int isBoundary;

- (id)initWithSequence:(MusicSequence)aSequence;
- (MusicTimeStamp)valueForNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;    // returns a float value appropriate for aNode's tier and type (use for generation)
- (UInt32)valueBinForNode:(/*__unsafe_unretained*/ mcCMNode*)aNode;         // returns a quantization bin appropriate for aNode's tier and type (use for comparisons)
- (void)setTimestamp:(Float64)ts;
- (MusicTimeStamp)timestamp;
- (float)schema;                                    // encodes the schema value as float "-" = 1.0, "0" = 0.0, "+" = 2.0
- (float)invariance;                                // queries the invarianceType and returns the appropriate value (interval, scale step, etc.)
- (float)identity;                                  // queries the identityType and returns the appropriate value
- (float)rSchema;                                   // encodes rSchema value as float (same as above)
- (float)rInvariance;                               // queries rInvarianceType and returns appropriate value
- (float)rIdentity;                                 // queries rIdentityType and returns appropriate value
- (UInt32)schemaBinForModel:(/*__unsafe_unretained*/ mcCMModel*)aModel;      // returns the bin index for the event's schema rep given model
- (UInt32)invarianceBinForModel:(/*__unsafe_unretained*/ mcCMModel*)aModel;  // returns the bin index for the event's invariance rep given model
- (UInt32)identityBinForModel:(/*__unsafe_unretained*/ mcCMModel*)aModel;    // returns the bin index for the event's identity rep given model
- (UInt32)MIDIPitch;                                // a convenient accessor for musicEvent->pitch
- (char)contour;                                    // calculated pitch contour
- (char)rContour;                                   // calculated rhythmic contour
- (int)interval;                                    // these are all calculated from prevEvent, not stored
- (float)beatED;
- (long)ms_timestamp;
- (long)ms_IOI;
- (float)IOIinBeats;
- (float)IOIRatio;
- (long)ms_duration;
- (long)ms_OOI;
- (float)frequency;
- (float)frequencyRatio;

@end
