//
//  mcPerceptionModule.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-11.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "mcEvent.h"

@class mcMusiCogController, mcWMStream, mcWMSegment, mcEvent, mcGroup, ADMMController;

@interface mcPerceptionModule : NSObject

@property (weak, nonatomic) mcMusiCogController* musiCog;
@property (strong, nonatomic) mcGroup* PGroup;
@property (strong, nonatomic) NSMutableArray* activeEvents;
@property (strong, nonatomic) NSMutableArray* feedbackGroupBuffer;
@property (assign, nonatomic) float* IOIRatioBins;
@property (assign, nonatomic) float* IOIRatioWeights;
@property (assign, nonatomic) BOOL acceptLiveInput;
@property (assign, nonatomic) float* runningChroma;
@property (assign, nonatomic) float* runningMetricAutocorrelation;
@property (assign, nonatomic) float* runningMetricInference;

// ---- want to test the ADMM idea
@property (strong, nonatomic) ADMMController* admmController;

+ (float*) strengthOfReturnToMean;
+ (float*) ionian;
+ (UInt8*) modeRotations;
+ (float) calculateRhythmicProximityFromEventA:(mcEvent*)eventA toEventB:(mcEvent*)eventB isGenerating:(BOOL)flag;
+ (float) calculateRhythmicSeparationFromEventA:(mcEvent*)eventA toEventB:(mcEvent*)eventB isGenerating:(BOOL)flag;
+ (float) calculatePitchExpectancyFromEventA:(mcEvent*)eventA toEventB:(mcEvent*)eventB inStream:(mcWMStream*)aStream;
+ (float) calculateRhythmicExpectancyFromEventA:(mcEvent*)eventA toEventB:(mcEvent*)eventB;
+ (float) calculateStrengthOfReturnToMeanForNote:(mcEvent*)aNote givenPreviousNote:(mcEvent*)prevNote inStream:(mcWMStream*)aStream;
+ (NSArray*) estimateModeAndTonalityForStream:(mcWMStream*)aStream withConfidence:(float*)conf;
+ (UInt8) estimateModeForNoteArray:(NSArray*)notes
                        fromStream:(mcWMStream *)aStream
                        withChroma:(float *)chroma
                        confidence:(float *)conf
                       useSalience:(BOOL)flag;
+ (UInt8) estimateModeForSegment:(mcWMSegment*)aSegment withChroma:(float *)chroma;
+ (NSArray*) estimateModeAndTonalityForSegment:(mcWMSegment *)aSegment withConfidence:(float*)conf;
+ (float) confidenceInMajorKeyForChroma:(float*)chroma andTonicPitchClass:(int)tonic;
+ (float) confidenceInMinorKeyForChroma:(float *)chroma andTonicPitchClass:(int)tonic;
+ (void) chroma:(float*)buffer forEventArray:(NSArray *)notes useSalience:(BOOL)flag;
+ (int) countUniquePitchClassesInNoteArray:(NSArray*)notes;
+ (float) incurCostForVoiceCrossingWithStream:(mcWMStream*)thisStream
                                 candidateNote:(mcEvent*)aNote
                           andCompetingStreams:(NSMutableArray*)streams;

- (void) normalizedRunningChroma:(float*)chromaVector;
- (void) normalizedRunningMetricInference:(float*)metricInf;

- (void) passMIDIData:(NSArray*)midiData;
- (void) performVoiceSeparationOnEventsInGroup:(mcGroup*)aGroup;
- (void) addFeedbackEventToFeedbackGroupBuffer:(MCMusicEvent*)feedbackEvent;
- (void) identifyRestingStreams;
- (void) matchEvents:(NSMutableArray*)tempEvents toStreams:(NSMutableArray*)tempStreams;
- (float) calculateCostFromEventA:(mcEvent*)eventA
                                   toEventB:(mcEvent*)eventB
                                withSegment:(mcWMSegment*)aSegment
                                 fromStream:(mcWMStream*)aStream
                       andConcurrentStreams:(NSMutableArray*)streams;
- (void) calculatePitchSalienceOfNotesInGroup:(mcGroup*)aGroup;
- (void) scalePitchSalienceOfInnerVoices:(NSArray*)sortedEvents;                     // NOTE: This takes a "chord" (or Group)
- (void) scalePitchSalienceOfHarmonicallySupportedVoices:(NSArray*)sortedEvents;
- (void) updateRunningChroma;
- (void) updateMetricAutocorrelation;
- (UInt16) getPeriodicityFromVector:(float*)aVector;
- (NSArray*) getCurrentMetricPeriodAndPosition;
- (id) initWithMusiCog:(mcMusiCogController*)mc;

- (void) updateIOIRatioGrid:(float)IOIRatio;
- (float) lockToIOIRatioGrid:(float)IOIRatio;
- (UInt16) IOIRatioBinIndexFromIOIRatio:(float)IOIRatio;

- (void) inputPitch:(UInt32)aPitch
       withVelocity:(UInt8)aVelocity
          onChannel:(UInt8)aChannel
             atTime:(Float64)aTimestamp;   // beat-based timestamp

- (void) buildPGroup;
- (void) debounce;

@end

enum {
	kMidiMessage_ControlChange     = 0xB0,
	kMidiMessage_ProgramChange     = 0xC0,
	kMidiMessage_BankMSBControl   = 0x00,
	kMidiMessage_BankLSBControl    = 0x32,
	kMidiMessage_NoteOn       = 0x90,
	kMidiMessage_NoteOff	= 0x80
};
