//
//  mcProductionModule.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-09.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "mcPlanner.h"

@class mcPMTexture, mcCMMotive, mcCMModel, mcCMNode, mcWMStream, mcWMSegment, mcPMController, mcEvent;

@interface mcProductionModule : NSObject

@property (weak, nonatomic) mcPMController* controller;
@property (strong, nonatomic) NSMutableArray* generationQueue;
@property (strong, nonatomic) mcPMTexture* feedbackGenerationTexture;
@property (strong, nonatomic) NSMutableArray* generationStreams;
@property (strong, nonatomic) NSArray* targetModeAndTonality;
@property (strong, nonatomic) mcWMSegment* lastGeneratedSegment;

- (BOOL) updateLevelStartMotive:(mcCMMotive*)aMotive
    usingPlanFromSubLevelNumber:(UInt16)levelNum
                       forModel:(mcCMModel*)aModel
                      andStream:(mcWMStream*)aStream;
- (BOOL) updateLevelStartMotive:(mcCMMotive*)aMotive
   usingStateFromSubLevelNumber:(UInt16)levelNum
                       forModel:(mcCMModel*)aModel
                      andStream:(mcWMStream*)aStream;
- (BOOL) updateLevelStartMotive:(mcCMMotive*)aMotive
  usingWMContentsForLevelNumber:(UInt16)levelNum
                       forModel:(mcCMModel*)aModel
                      andStream:(mcWMStream*)aStream;
- (BOOL) updateLevelStartMotive:(mcCMMotive *)aMotive
     usingWMAssociationForModel:(mcCMModel *)aModel
                      andStream:(mcWMStream *)aStream;
- (BOOL) updateLevelStartMotive:(mcCMMotive*)aMotive
    probabilisticallyForLevelNumber:(UInt16)levelNum
                       forModel:(mcCMModel*)aModel
                      andStream:(mcWMStream*)aStream;
- (BOOL) extendPlanFromStartMotive:(mcCMMotive*)aMotive
                         forStream:(mcWMStream*)aStream;
- (NSMutableArray*) extractMotivesFromWM;
- (void) repairInvalidL2Motive:(mcCMMotive*)aMotive forStream:(mcWMStream*)aStream;
- (void) ensureTierMatchesPlanningTierForMotive:(mcCMMotive*)aMotive;
- (mcWMSegment*) generateOutputSegmentWithoutContextFromL2Motive:(mcCMMotive *)aMotive;
- (mcWMSegment*) generateOutputSegmentWithSeedSegment:(mcWMSegment *)seedSegment fromL2Motive:(mcCMMotive *)aMotive;
- (mcWMSegment*) generateFinalBoundaryForEndingWithSeedSegment:(mcWMSegment*)seedSegment fromL2Motive:(mcCMMotive*)aMotive;
- (enum mcPlanningStatus) continueStream:(mcWMStream*)aStream;
- (void) forwardFillRhythmBranch:(NSMutableArray *)rBranch toLength:(UInt16)length atTier:(UInt16)tier;
- (NSArray*) getRhythmBranchByAssociationWithPitchTerminal:(mcCMEdge*)pitchTerm;
- (NSArray*) getRhythmBranchForLengthOfPitchBranch:(NSArray*)pBranch;
- (NSMutableDictionary*) buildGroupsListFromGenerationQueue;
- (void) matchPlanningTierForAllTerminalEdges:(NSMutableArray*)terminalEdges;
- (void) setValueForEvent:(mcEvent*)anEvent withNode:(mcCMNode *)aNode andSeed:(mcEvent *)aSeed;
- (void) renderTexture;
- (void) transformSegment:(mcWMSegment *)aSegment
       withContextSegment:(mcWMSegment *)context
  toTargetModeAndTonality:(NSArray *)tModeTonality
                forStream:(mcWMStream *)aStream;
- (void) enqueueGenerationEvent:(mcEvent *)anEvent;
- (void) checkFeedbackEventAgainstGenerationQueue:(mcEvent *)feedbackEvent;
- (id) initWithController:(mcPMController*)aController;

@end
