/*
 *  musicUtils.h
 *  ManuScore
 *
 *  Created by James Maxwell on 10-09-14.
 *  Copyright 2010 rubato-music.com. All rights reserved.
 *
 */

#include <AudioToolbox/AudioToolbox.h>


// ----------- Distance/Cost Measures ------------- //

float pitchSalienceByRegister(int pitch);

float scalingByRhythmicSeparation(int separationLength);

float rhythmicProximityByIOI(int IOI);

float pitchABProximity(int pitchA, int pitchB);

float pitchABSimilarity(int pitchA, int pitchB);

float intervalABProximity(int intervalA, int intervalB);

float intervalABSimilarity(int intervalA, int intervalB);

float beatEDSimilarity(float beatEDA, float beatEDB);

float EDSimilarity(float edA, float edB);

float EDRatioSimilarity(float edRatA, float edRatB);

float zeroToOneSigmoid(float x);

float zeroToNSigmoid(float x, float max);

bool chromaticismOfChromaVector(float* chroma);

float slopeFromOneToZOverTime(float x, float z, float time);

float gaussianWithPeakAndWidth(double x, double peak, double width);

float gaussianWithPeakWidthAndOffset(double x, double peak, double width, double offset);

float inverseGaussianWithMinWidthAndOffset(double x, double min, double width, double offset);

float sigmoidWithLowerLimitAndSlope(double x, double lowLim, double slope);

double weightedCombinationOfValuesWithHighDynamicBias(double x, double y);

double weightedCombinationOfValuesWithLowDynamicBias(double x, double y);

double weightedCombinationOfValuesBiasedTowardX(double x, double y);

float weightedCombinationOfXandYWithLinearBiasTowardX(float x, float y);

float msFromBeatAtTempo(float beat, float tempo);

float truncateToMaxBeats(float input, float max);

float MIDIToFreq(int pitch);

float degreesToRadians(float degrees);

float roundTimestampToExpectancyBasedEighthNote(float timestamp);

void convertToIntervalClassVector(int *icVector, int *pitchVector, int length);

MusicTimeStamp getSequenceLength(MusicSequence aSequence);

//----------- Inter-Onset Beat encode/decode --------------------- //

float encodeBeatEntryDelay(float prevTimestamp, float timestamp);
float decodeBeatEntryDelay(float prevTimestamp, float IOB);

// --------- ED ratio decode ------------------------------- //

float timestampFromPreviousTimestampAndEDRatio(float prevTimestamp, float prevED, float EDRatio);

// ---------- Utilities for Transform Functions ---------------- //

// --------- Rhythm expectancy functions ------------------- //

float rhythmicExpectancyByIOIRatio(float IOIRatio);

double rhythmicExpectancyByIOI(double IOI);

float metricClosure(float beatTick, int numerator, int denominator);

// ----------- float to fraction using continuous fractions ----------- //

typedef struct Frac {
	int top;
	int bottom;
	float value;
	float error;
} Frac;

Frac scontinued_fraction(int limit, float val);

Frac bestFractionForFloat(float val, float tolerance);

// ------------- musicRep struct for Hatches and Pitches ------------ //

// new MnSNote ED struc
typedef struct edInfo {
    long long int beatEDBinIndex;
    long long int edRatioBinIndex;
	long long int seconds;
	long long int EDBinIndex;
} edInfo;

// new musicRep struct
typedef struct musicRep {
	long long int channel;
	long long int pitch;
	long long int velocity;
	float timestamp;
	float ED;
	float duration;
	long long int meta1;
	long long int meta2;
	long long int bend;
} musicRep;